package ru.ifmo.vizi.dmkm;

import ru.ifmo.vizi.base.Configuration;
import ru.ifmo.vizi.base.widgets.ShapeStyle;
import ru.ifmo.vizi.dmkm.net.Net;
import ru.ifmo.vizi.dmkm.net.VertexLayouter;
import ru.ifmo.vizi.dmkm.widgets.Arc;
import ru.ifmo.vizi.dmkm.widgets.Vertex;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
 * The instance of this class provides user to edit the network.
 * @author Yuri Bedny
 * @version 0.0
 */
public class NetEditor implements MouseListener, MouseMotionListener {

    /**
     * Constans for notify the listeners.
     */
    private final int NET_CHANGED_EVENT = 0;
    private final int ARC_DRAGED_EVENT = 1;
    private final int MENU_POPUP_EVENT = 2;

    private DMKMVisualizer applet;

    private int maxArcCapacity;

    private Arc arc;

    /**
     * Whether editor is active.
     */
    private boolean isActive;

    private boolean isDragged;

    /**
     * Vizi's configuration.
     */
    private Configuration config;

    /**
     * The net.
      */
    private Net net;

    /**
     * Adjacency matrix.
     */
    private int g[][];

    /**
     * Layout manager.
     */
    private VertexLayouter layouter;

    /**
     * Styles for arc.
     */
    private ShapeStyle arcStyles[];

    /**
     * Number of the pressed vertices.
     */
    private int i, j;

    /**
     * Popup menu.
     */
    private EditMenu menu;

    /**
     * Returns true is editor is active.
     * @return true if editor is active; false otherwise
     */
    public boolean isActive() {
        return isActive;
    }

    /**
     * Notifies the listeners.
     */
    private void notifyApplet(int eventNumber) {
        if (eventNumber == NET_CHANGED_EVENT)
            applet.onNetChanged();
        else if (eventNumber == ARC_DRAGED_EVENT)
            applet.onArcDragged();
        else if (eventNumber == MENU_POPUP_EVENT) applet.onMenuPopUp();
    }

    /**
     * Ands an arc between vertices i and j.
     */
    private void addArc(int i, int j) {
        if (i == -1 || j == -1 || i == j || g[i][j] > 0 || g[j][i] > 0) return;
        g[i][j] = (int) (1 + Math.random() * maxArcCapacity);
        notifyApplet(NET_CHANGED_EVENT);
    }

    public void mousePressed(MouseEvent me) {
        isDragged = true;
        i = layouter.getVertexNumber(me.getPoint());
    }

    public void mouseReleased(MouseEvent me) {
        isDragged = false;
        net.remove(arc);
        notifyApplet(ARC_DRAGED_EVENT);
        j = layouter.getVertexNumber(me.getPoint());
        addArc(i, j);
    }

    private void getStyles() {
        arcStyles = ShapeStyle.loadStyleSet(config, "edit-arc");
   }

    /**
     * Sets the net.
     * @param net current net.
     */
    public void setNet(Net net) {
        this.net = net;
    }

    /**
     * Creates net editor with given configuration.
     * @param config vis's configuration.
     */
    public NetEditor(Configuration config, DMKMVisualizer applet) {
        this.applet = applet;
        this.config = config;
        getStyles();
        maxArcCapacity = config.getInteger("maxArcCapacity");
        menu = new EditMenu(config, this);
        arc = new Arc(arcStyles);
    }

    /**
     * Sets editor to passive mode.
     */
    public void setPassive() {
        isActive = false;
    }

    /**
     * Sets editor to active mode.
     * @param g current capacity.
     * @param layouter vertex layouter for current net.
     */
    public void setActive(int g[][], VertexLayouter layouter) {
        isActive = true;
        this.g = g;
        this.layouter = layouter;
    }

    /**
     * Adds an acr then mouse dragged.
     */
    public void mouseDragged(MouseEvent me) {
        if (i == -1) return;
        Point p1 = layouter.getVertexCenter(i);
        Point p2 = me.getPoint();
        net.remove(arc);
        j = layouter.getVertexNumber(p2);
        if (j == i) {
            notifyApplet(ARC_DRAGED_EVENT);
            return;
        }
        int diam = layouter.getVertexDiameter();
        if (j != -1) {
            arc = new Arc(arcStyles, diam, diam, p1, layouter.getVertexCenter(j));
            arc.setStyle(0);
        } else {
            arc = new Arc(arcStyles, diam, 0, p1, p2);
            arc.setStyle(1);
        }
        arc.setSize(net.getSize());
        int minSize = Math.min(net.getSize().width, net.getSize().height);
        arc.setHeadSize(
                (int) (minSize * config.getDouble("arc-length-percent") / 100),
                (int) (minSize * config.getDouble("arc-width-percent") / 100)
        );

        net.add(arc);
        notifyApplet(ARC_DRAGED_EVENT);
    }

    private void checkPlusMinusEnabled(int i, int j) {
        if (g[i][j] == 1) {
            menu.setMinusEnabled(false);
        } else {
            menu.setMinusEnabled(true);
        }
        if (g[i][j] == maxArcCapacity) {
            menu.setPlusEnabled(false);
        } else {
            menu.setPlusEnabled(true);
        }
    }

    /**
     * Adds value d to capacity of the arc between vertices i and j.
     * @param i the startPoint vertex.
     * @param j the secondPoint vertex.
     * @param d additional value.
     */
    public void changeArc(int i, int j, int d) {
        g[i][j] += d;
        if (g[i][j] < 1) g[i][j] = 1;
        checkPlusMinusEnabled(i, j);
        notifyApplet(NET_CHANGED_EVENT);
    }

    /**
     * Removes arc between vertices i and j from net.
     * @param i the startPoint vertex.
     * @param j the secondPoint vertex.
     */
    public void deleteArc(int i, int j) {
        g[i][j] = 0;
        net.remove(menu);
        notifyApplet(NET_CHANGED_EVENT);
    }


    private double dist(Point p1, Point p2) {
        double dx = p2.x - p1.x;
        double dy = p2.y - p1.y;
        return Math.sqrt(dx * dx + dy * dy);
    }

    /**
     * Hides menu.
     */
    public void mouseMoved(MouseEvent me) {
        if (!(me.getSource().equals(net)) || menu.getCircle() == null) return;
        Point mp = me.getPoint();
        Point cp = menu.getCircle().getCenter();
        if (dist(mp, cp) > dist(cp, menu.getLocation()) || (mp.x - cp.x < 0)) {
            net.remove(menu);
            net.repaint();
        }
    }

    //  x      
    public void mouseEntered(MouseEvent me) {
        if (me.getSource() instanceof Net || isDragged) return;
        Vertex cir = (Vertex) me.getSource();
        menu.setArc(cir.getV1(), cir.getV2());
        checkPlusMinusEnabled(cir.getV1(), cir.getV2());
        int width = (int) (Math.pow(10 * cir.getRadius(), 0.6));
        menu.setSize(width, 3 * width);
        menu.setLocation(cir.getLocation().x + cir.getRadius() * 2 + 2, cir.getLocation().y + cir.getRadius() - menu.getSize().height / 2);
        menu.resizeButtons();
        menu.setCircle(cir);
        net.add(menu);
        notifyApplet(MENU_POPUP_EVENT);
    }

    /**
     * Invoked when vis data changed.
     * @param g capacity.
     */
    public void updateAppletData(int g[][]) {
        this.g = g;
    }

    public void mouseClicked(MouseEvent me) {
    }

    public void mouseExited(MouseEvent me) {

    }

}
