package ru.ifmo.vizi.dmkm.net;

import java.awt.*;

/**
 * A vertex layouter lays out vertices.
 * @author Yuri Bedny
 * @version 0.0
 */
public class VertexLayouter {

    /**
     * Number of vertices in net.
     */
    private int n;

    /**
     * Diameter of the vertex.
     */
    private int diam;

    /**
     * Width of the client area.
     */
    private int width;

    /**
     * Height of the client area.
     */
    private int height;

    /**
     * Sets size of client area for painting.
     * @param size new size to set.
     */
    public void setSize(Dimension size) {
        width = size.width;
        height = size.height;
    }

    /**
     * Sets the number of vertices in net.
     * @param n number of vertices to set.
     */
    public void setNumOfVertices(int n) {
        this.n = n;
    }

    /**
     * Creates vertex layouter with specified number of vertices in network and diameter of vertex.
     * @param n number of vertices.
     * @param diam diameter of vertex.
     */
    public VertexLayouter(int n, int diam) {
        this.n = n;
        this.diam = diam;
    }

    /**
     * The distance between specified points.
     * @param p1 the first point.
     * @param p2 the second point.
     * @return the distance between specified points.
     */
    private double dist(Point p1, Point p2) {
        double dx = p2.x - p1.x;
        double dy = p2.y - p1.y;
        return Math.sqrt(dx * dx + dy * dy);
    }

    /**
     * Returns the number of vertex by it's location.
     * @param p vertex location.
     * @return the number of vertex.
     */
    public int getVertexNumber(Point p) {
        for (int i = 0; i < n; i++) {
            double x = getLocation(i).x + diam / 2;
            double y = getLocation(i).y + diam / 2;
            if (dist(p, new Point((int)x, (int)y)) < diam / 2)
                return i;
         }

        return -1;
    }

    /**
     * Returns the center of vertex location.
     * @param i number of vertex.
     * @return the center of vertex location.
     */
    public Point getVertexCenter(int i) {
        Point p = getLocation(i);
        return new Point(p.x + diam / 2, p.y + diam / 2);
    }

    /**
     * Returns diameter of vertex.
     * @return diameter of vertex.
     */
    public int getVertexDiameter() {
        return diam;
    }

    public void setVertexDiameter(int diam) {
        this.diam = diam;
    }

    /**
     * Returns vertex location (left-top coner) by it's number.
     * @param i number of vertex.
     * @return vertex location.
     */
    public Point getLocation(int i) {
        int x, y;
        double size = diam;
        double delta = height / 10;
        double d = height / 4 - size / 2;
        double w = (width - 2 * d - 2 * size);
        int n1 = (int)Math.ceil(((double)n - 2) / 2);
        int n2 = (int)Math.floor(((double)n - 2) / 2);
        double dx1 = w / n1;
        double dx2 = w / n2;
        if (i == 0) {
            x = (int)(d - delta);
            y = (int)(height / 2 - size / 2);
        } else if (i == n - 1) {
            x = (int)(width - size - d + delta);
            y = (int)(height / 2 - size / 2);
        } else if (i <= n1) {
            x = (int)((d + size) + dx1 * i - dx1 / 2 - size / 2);
            y = (int)(height / 4 - size / 2 - delta);
        } else {
            x = (int)((d + size) + dx2 * (i - n1) - dx2 / 2 - size / 2);
            y = (int)(height * 3 / 4 - size / 2 + delta);
        }
        if (n == 7) {
            if (i == 1 || i == 3) y += delta;
            if (i == 1) x -= delta;
            if (i == 3) x += delta;
        } else if (n == 8) {
            if (i == 1 || i == 3) y += delta;
            if (i == 4 || i == 6) y -= delta;
            if (i == 1 || i == 4) x -= delta;
            if (i == 3 || i == 6) x += delta;
        } else if (n == 9) {
            if (i == 1 || i == 4) y += delta;
            if (i == 5 || i == 7) y -= delta;
            if (i == 1 || i == 5) x -= delta;
            if (i == 4 || i == 7) x += delta;
        } else if (n == 10) {
            if (i == 1 || i == 4) y += delta;
            if (i == 5 || i == 8) y -= delta;
            if (i == 1 || i == 5) x -= delta;
            if (i == 4 || i == 8) x += delta;
        }
        return new Point(x, y);
    }

}
